<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * The Gadget WDK empowers you with an thight integrated infrastructure for web 
 * development.  It is based on Widgets and borrows most of its power from PEAR 
 * by using HTML_QuickForm, I18Nv2, several HTTP packages and DB_DataObject 
 * resp. DB (see dependencies).  Many important services like authentication, 
 * rights management and error handling as well as providers for "current user",
 * preferences and configuration can be instantly replaced.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Widget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.2 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Pager.php,v 1.2 2005/04/15 13:07:46 mike Exp $
 */

/**
 * Requires Gadget_Formatter
 */
class_exists('Gadget_Widget') or require_once 'Gadget/Widget.php';

/**
 * Gadget_Pager
 *
 * Provides paging routines.
 * 
 * Template Example:
 * <code>
 * <p class="center">
 * 	{if: $hasPrev}
 * 	    <a href="{$widget->href($prev)}">&laquo; forward</a>
 * 	{else}
 * 	    &laquo; forward
 * 	{end}
 * 	<span>|</span>
 * 	{foreach: $moveLinks as $l}
 * 		{if: $l->active}
 * 			{$l->num}
 * 		{else}
 * 			<a href="{$widget->href($l->offset)}">{$l->num}</a>
 * 		{end}
 * 	<span>|</span>
 * 	{end}
 * 	{if: $hasNext}
 * 	    <a href="{$widget->href($next)}">back &raquo;</a>
 * 	{else}
 * 	    back &raquo;
 * 	{end}
 * </p>
 * </code>
 * 
 * @package     Gadget
 * @subpackage  Gadget_Widget
 * @since       1.0
 */
class Gadget_Pager extends Gadget_Widget
{
    /**
     * Offset of previous page
     * 
     * @var int
     */
    var $prev = 0;
    
    /**
     * Offset of next page
     * 
     * @var int
     */
    var $next = 0;
    
    /**
     * Has previous page
     * 
     * @var bool
     */
    var $hasPrev = false;
    
    /**
     * Has next page
     * 
     * @var bool
     */
    var $hasNext = false;
    
    /**
     * Current Offset
     * 
     * @var int
     */
    var $offset = 0;
    
    /**
     * Limit
     * 
     * @var int
     */
    var $limit = 10;
    
    /**
     * Move Links
     * 
     * @var array
     */
    var $moveLinks = array();
    
    /**
     * Whether to do anything if page() is called
     * 
     * @var bool
     */
    var $page = true;
    
    /**
     * Load Gadget_Formatter by default
     * 
     * @var bool
     */
    var $aggregateFormatter = true;
    
    /**
     * On Before Construct
     * 
     * @access  protected
     * @return  void
     */
    function onBeforeConstruct()
    {
        // set offset
        $this->offset = Gadget::getQ('o', 'int', 0, true);
        
        // set limit
        $this->limit = $this->getLimit();
        
        // parent's onBeforeConstruct()
        parent::onBeforeConstruct();
    }
    
    /**
     * On After Construct
     * 
     * @access  protected
     * @return  void
     */
    function onAfterConstruct()
    {
        // fix offset
        if ($diff = $this->offset % $this->limit) {
            $this->offset -= $diff;
        }
        parent::onAfterConstruct();
    }
    
    /**
     * Get Limit
     * 
     * Override this in your Widget to return a custom limit.
     * 
     * @access  public
     * @return  int
     */
    function getLimit()
    {
        return $this->limit;
    }
    
    /**
     * Page
     * 
     * This method must be called with the total count of items.
     * 
     * @access  public
     * @return  void
     * @param   int     $total
     */
    function page($total)
    {
        if ($this->page) {
            $this->makePrevNext($total);
            $this->makeMoveLinks($total);
            $this->makeMetaLinks($total);
        }
    }
    
    /**
     * Href
     * 
     * @access  public
     * @return  string
     * @param   int     $offset
     * @param   bool    $amp
     */
    function href($offset, $amp = true)
    {
        $qs = $this->queryString .'&o='. $offset;
        return $this->location .'?'. ($amp ? htmlspecialchars($qs) : $qs);
    }
    
    /**
     * Make Prev and Next Calculations
     * 
     * @access  protected
     * @return  void
     * @param   int     $total
     */
    function makePrevNext($total)
    {
        $this->prev = $this->offset - $this->limit;
        $this->next = $this->offset + $this->limit;
        
        $this->hasPrev = $this->prev >= 0;
        $this->hasNext = $this->next < $total;
    }
    
    /**
     * Make Meta Links
     * 
     * @access  protected
     * @return  void
     * @param   int     $total
     */
    function makeMetaLinks($total)
    {
        $header = &Gadget::header();
        if ($this->hasPrev) {
            $header->addLink('previous', $this->href($this->prev));
        }
        if ($this->hasNext) {
            $header->addLink('next', $this->href($this->next));
        }
        if ($this->offset) {
            $header->addLink('first', $this->href(0));
        }
        
        if ($diff = $total % $this->limit) {
            $last = $total - $diff;
        } else {
            $last = $total - $this->limit;
        }
        
        if ($last > $this->offset) {
            $header->addLink('last', $this->href($last));
        }
        unset($header);
    }
    
    /**
     * Make Move Links
     * 
     * @access  protected
     * @return  void
     * @param   int     $total
     */
    function makeMoveLinks($total)
    {
        // calculate min; the beginning
        $max = 0;
        $min = $this->offset - (4 * $this->limit);
        
        // check min >= 0
        if ($min < 0) {
            $max = abs($min);
            $min = 0;
        }
        
        // calculate max; the end
        $max += $this->offset + (4 * $this->limit);
        
        // if min !=0 and max > foundRows sub diff from min
        if ($min && $max > $total) {
            $diff = $max - $total;
            $min -= $diff + ($this->limit - $diff % $this->limit);
        }
        
        // re-check min >= 0
        if ($min < 0) {
            $max += abs($min);
            $min = 0;
        }
        
        // where do we start
        $cnt = $min ? $min / $this->limit : 0;
        
        for ($o = $min; $o < $total && $o <= $max; $o += $this->limit) {
            $this->moveLinks[] = (object) array(
                'num'   => ++$cnt,
                'offset'=> $o,
                'active'=> $o == $this->offset,
            );
        }
    }
}

?>
